@@ -1,4 +1,4 @@
-# Copyright (C) 2023,2024 Giovanni Fulco
+# Copyright (C) 2023,2024,2025 Giovanni Fulco
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,9 +16,11 @@
 from option_key import OptionKey
 
 
-def get_option(options : dict[str, any], option_key : OptionKey) -> any:
-    return options[option_key.get_name()] if option_key.get_name() in options else option_key.get_default_value()
+def get_option(options: dict[str, any], option_key: OptionKey) -> any:
+    return (options[option_key.get_name()]
+            if options and option_key.get_name() in options
+            else option_key.get_default_value())
 
 
-def set_option(options : dict[str, any], option_key : OptionKey, option_value : any) -> None:
+def set_option(options: dict[str, any], option_key: OptionKey, option_value: any) -> None:
     options[option_key.get_name()] = option_value
